/*********************************************************************
                                                        version 0.04
    Windows DIBitmap Input Class Library - Implement
            Copyright (c) S.Chiba 1994, 1995, 1996

*********************************************************************/
#include "stdafx.h"
#include "CDIB.h"

#undef SUPPORTED8BIT
#define BITMAPFILEHEADER_TYPE ('M' << 8 | 'B')

inline DWORD DIBBITPALCOUNT( int BitCount, int Compression )
{
	if ( BitCount <= 8 ) return (1 << BitCount);
	else if ( (BitCount == 32 || BitCount == 16) && Compression == BI_BITFIELDS ) return 3;
	else return 0;
}

CDIB::CDIB()
{
	m_bmiHeader = NULL;
}

CDIB::~CDIB()
{
	Destroy();
}

BOOL CDIB::Create( LONG w, LONG h, int bitcount, int compression )
{
	if ( compression != BI_RGB && compression != BI_BITFIELDS ) return FALSE;
#ifdef SUPPORTED8BIT
	if ( bitcount != 8 && bitcount != 24 ) return FALSE;
#else
	if ( bitcount != 24 ) return FALSE;
#endif

	Destroy();

	DWORD ImageSize = ((w * bitcount + 31) & ~31) / 8 * h;
	DWORD ColorTbl = (bitcount <= 8) ? (1 << bitcount) : 0;
	DWORD MemorySize = (DWORD) sizeof(BITMAPINFOHEADER) + (DWORD) sizeof(RGBQUAD) * ColorTbl + ImageSize;
	m_bmiHeader = (LPBITMAPINFOHEADER) new BYTE[MemorySize];
	if ( ! m_bmiHeader ) return FALSE;

	m_bmiHeader->biSize          = sizeof(BITMAPINFOHEADER);
	m_bmiHeader->biWidth         = w;
	m_bmiHeader->biHeight        = h;
	m_bmiHeader->biPlanes        = 1;
	m_bmiHeader->biBitCount      = bitcount;
	m_bmiHeader->biCompression   = compression;
	m_bmiHeader->biSizeImage     = ImageSize;
	m_bmiHeader->biXPelsPerMeter = 0;
	m_bmiHeader->biYPelsPerMeter = 0;
	m_bmiHeader->biClrUsed       = ColorTbl;
	m_bmiHeader->biClrImportant  = 0;

	return TRUE;
}

void CDIB::Destroy()
{
	if ( m_bmiHeader ) delete m_bmiHeader;
	m_bmiHeader = NULL;
}

void CDIB::Serialize( CArchive& ar )
{
	CFile *cfile = ar.GetFile();
	if ( ! ar.IsStoring() )
	{
		LoadFile( cfile );
	}
}

BOOL CDIB::LoadFile( CFile *cfile )
{
	BITMAPFILEHEADER filehead;
	BITMAPINFOHEADER infohead;

	Destroy();

	cfile->SeekToBegin();
	if ( cfile->Read( &filehead, sizeof(BITMAPFILEHEADER) ) != sizeof(BITMAPFILEHEADER) )
	{
		AfxThrowArchiveException( CArchiveException::endOfFile, NULL );
		ASSERT(0);
	}
	if ( filehead.bfType != BITMAPFILEHEADER_TYPE )
	{
		AfxThrowArchiveException( CArchiveException::badIndex, NULL );
		ASSERT(0);
	}

	if ( cfile->Read( &infohead, sizeof(BITMAPINFOHEADER) ) != sizeof(BITMAPINFOHEADER) )
	{
		AfxThrowArchiveException( CArchiveException::endOfFile, NULL );
		ASSERT(0);
	}
	if ( infohead.biSize != sizeof(BITMAPINFOHEADER) )
	{
		AfxThrowArchiveException( CArchiveException::badIndex, NULL );
		ASSERT(0);
	}
#ifdef SUPPORTED8BIT
	if ( infohead.biBitCount != 8 && infohead.biBitCount != 24 )
#else
	if ( infohead.biBitCount != 24 )
#endif
	{
		AfxThrowArchiveException( CArchiveException::badIndex, NULL );
		ASSERT(0);
	}

	if ( ! (m_bmiHeader = (LPBITMAPINFOHEADER) new BYTE[filehead.bfSize]) )
	{
		AfxThrowMemoryException();
		ASSERT(0);	
	}
	CopyMemory( m_bmiHeader, &infohead, sizeof(BITMAPINFOHEADER) );

	UINT LoadSize = filehead.bfSize - sizeof(BITMAPFILEHEADER) - sizeof(BITMAPINFOHEADER);
	LPVOID pBits = (LPVOID) ((LPSTR) m_bmiHeader + sizeof(BITMAPINFOHEADER));
	if ( cfile->Read( pBits, LoadSize ) != LoadSize )
	{
		Destroy();
		AfxThrowArchiveException( CArchiveException::endOfFile, NULL );
		ASSERT( 0 );
	}
	return TRUE;
}

BOOL CDIB::DoDraw( CDC* pDc, CRect& destRC, CPoint& offset /* = CPoint(0,0) */, int zoom /* = 100 */, DWORD dwRop /* = SRCCOPY */ )
{
	BOOL rslt = FALSE;
	HPALETTE hOldPal = NULL;

	if ( ! m_bmiHeader ) return FALSE;

	if ( zoom < 1 ) zoom = 1;
	CRect srcRC( CPoint( (destRC.left * 100 / zoom), (destRC.top * 100 / zoom) ), CSize( (destRC.Width() * 100 / zoom) + 2, (destRC.Height() * 100 / zoom) + 2) );
	CRect maxRC( CPoint( 0, 0 ), CSize( (int) GetWidth(), (int) GetHeight() ) );

	srcRC &= maxRC;
	if ( srcRC.IsRectEmpty() ) return FALSE;
	destRC = CRect( CPoint( offset.x, offset.y ), CSize( srcRC.Width() * zoom / 100, srcRC.Height() * zoom / 100) );

	HPALETTE hOld = NULL;
	HPALETTE hPal = GetDIBPalette();
	if ( hPal ) hOld = ::SelectPalette( pDc->m_hDC, hPal, TRUE );
	pDc->RealizePalette();

	pDc->SetStretchBltMode( COLORONCOLOR );
	rslt = ::StretchDIBits(
		pDc->m_hDC,					// Destnation DC
		destRC.left,
		destRC.top,
		destRC.Width(),
		destRC.Height(),			// Destnation Rect
		srcRC.left,
		(int) GetHeight() - srcRC.top - srcRC.Height(),
		srcRC.Width(),
		srcRC.Height(),				// Source Rect
		(LPVOID) GetBits(),        	// lpBits
		(LPBITMAPINFO) m_bmiHeader,	// lpBitsInfo
		DIB_RGB_COLORS,				// wUsage
		dwRop );					// ROP code
	if ( hOld ) ::SelectPalette( pDc->m_hDC, hOld, TRUE );
	return rslt != 0;
}

LONG CDIB::GetWidth()
{
	return m_bmiHeader ? m_bmiHeader->biWidth : 0;
}

LONG CDIB::GetHeight()
{
	return m_bmiHeader ? m_bmiHeader->biHeight : 0;
}

WORD CDIB::GetDepth()
{
	return m_bmiHeader ? m_bmiHeader->biBitCount : 0;
}

int CDIB::GetDIBPaletteCount()
{
	if ( ! m_bmiHeader ) return 0;
	return m_bmiHeader->biClrUsed ? m_bmiHeader->biClrUsed : DIBBITPALCOUNT( m_bmiHeader->biBitCount, m_bmiHeader->biCompression );
}

LONG CDIB::GetBitSize()
{
	return m_bmiHeader ? ((m_bmiHeader->biWidth + 3 & ~3) * m_bmiHeader->biHeight * (m_bmiHeader->biBitCount >> 3)) : 0;
}

LPVOID CDIB::GetBits()
{
	if ( ! m_bmiHeader ) return NULL;
	return (LPVOID) ((LPSTR) m_bmiHeader + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * GetDIBPaletteCount());
}

LPRGBQUAD CDIB::GetPalettes()
{
	return m_bmiHeader ? ((LPBITMAPINFO) m_bmiHeader)->bmiColors : NULL;
}

HPALETTE CDIB::GetDIBPalette()
{
	HPALETTE hPalette = NULL;
	if ( ! m_bmiHeader ) return NULL;

	LPRGBQUAD lpRGBQ = GetPalettes();
	WORD colorlen = (WORD) (m_bmiHeader->biClrUsed ? m_bmiHeader->biClrUsed : (1 << m_bmiHeader->biBitCount));

	if ( m_bmiHeader->biBitCount <= 8 )
	{
		LPLOGPALETTE lpPal = (LPLOGPALETTE) new BYTE[sizeof(LOGPALETTE) + sizeof(PALETTEENTRY) * colorlen];
		if ( lpPal ) {
			lpPal->palVersion = 0x300;
			lpPal->palNumEntries = colorlen;
			for( WORD i = 0; i < colorlen; i++ )
			{
				lpPal->palPalEntry[i].peRed   = lpRGBQ[i].rgbRed;
				lpPal->palPalEntry[i].peGreen = lpRGBQ[i].rgbGreen;
				lpPal->palPalEntry[i].peBlue  = lpRGBQ[i].rgbBlue;
				lpPal->palPalEntry[i].peFlags = PC_NOCOLLAPSE;
			}
			hPalette = ::CreatePalette( lpPal );
			delete lpPal;
		}
	}
	else hPalette = NULL;
	return hPalette;
}

BOOL CDIB::CopyPalette( LPVOID sciodibPal )
{
	if ( ! sciodibPal ) return FALSE;
	CopyMemory( sciodibPal, (LPBYTE) GetPalettes(), GetDIBPaletteCount() * 4 );
	return TRUE;
}

BOOL CDIB::CopyBits( LPVOID sciodibImage )
{
	if ( ! sciodibImage ) return FALSE;
	CopyMemory( sciodibImage, (LPBYTE) GetBits(), GetBitSize() );
	return TRUE;
}
