// KiffrefDoc.cpp : implementation of the CKiffrefDoc class
//

#include "stdafx.h"
#include "Kiffref.h"
#include "CDIB.h"

#include "KiffrefDoc.h"
#include "KiffrefView.h"
#include "KiffSettingsDlg.h"

#define USE_MEMORY 1

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CKiffrefDoc

IMPLEMENT_DYNCREATE(CKiffrefDoc, CDocument)

BEGIN_MESSAGE_MAP(CKiffrefDoc, CDocument)
	//{{AFX_MSG_MAP(CKiffrefDoc)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_ZOOM_50, OnZoom50)
	ON_COMMAND(ID_ZOOM_100, OnZoom100)
	ON_COMMAND(ID_ZOOM_200, OnZoom200)
	ON_UPDATE_COMMAND_UI(ID_ZOOM_50, OnUpdateZoom50)
	ON_UPDATE_COMMAND_UI(ID_ZOOM_100, OnUpdateZoom100)
	ON_UPDATE_COMMAND_UI(ID_ZOOM_200, OnUpdateZoom200)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKiffrefDoc construction/destruction

CKiffrefDoc::CKiffrefDoc()
{
	// TODO: add one-time construction code here

	m_zooming = 100;
	m_windowSize.cx = 400;
	m_windowSize.cx = 300;
	m_type = TYPE_NONE;

	hInst = LoadLibrary( "KIFF.DLL" );
	p_kiff = NULL;

	if ( ! hInst ) AfxMessageBox( "File not found - KIFF.DLL" );
	else
	{
		(FARPROC&) KiffProc = GetProcAddress( hInst, "KIFF_Constructor" );
		if ( KiffProc ) p_kiff = KiffProc();
		else AfxMessageBox( "GetProc error!" );
	}
}

CKiffrefDoc::~CKiffrefDoc()
{
	if ( p_kiff )
	{
		(FARPROC&) KiffFree = GetProcAddress( hInst, "KIFF_Destructor" );
		if ( KiffFree ) KiffFree( p_kiff );
		p_kiff = NULL;
	}
	if ( hInst )
	{
		FreeLibrary( hInst );
		hInst = NULL;
	}
}

BOOL CKiffrefDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	if ( ! p_kiff ) return FALSE;

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CKiffrefDoc serialization

void CKiffrefDoc::Serialize(CArchive& ar)
{
	WORD bm;
	DWORD bmsize;
	CFile *cfile = ar.GetFile();
	CString	string;

	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
		DWORD inport_len = cfile->GetLength();
		TRY
		{
			cfile->Read( &bm, 2 );
		}
		CATCH( CFileException, error )
		{
			return;
		}
		END_CATCH;
		TRY
		{
			cfile->Read( &bmsize, 4 );
		}
		CATCH( CFileException, error )
		{
			return;
		}
		END_CATCH;
		cfile->SeekToBegin();
		if ( bm == 'MB' && bmsize == inport_len )
		{
			m_DIB.LoadFile( cfile );
			m_type = TYPE_BITMAP;
		}
		else
		{
			cfile->Read( &bmsize, 4 );
			if ( bmsize == 'FFIK' )
			{
#if USE_MEMORY
				HANDLE hFile;
				LPVOID p;
				LONG size;
				DWORD dw;
				hFile = ::CreateFile( cfile->GetFilePath(), GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, NULL, NULL );
				size = (LONG) ::GetFileSize( hFile, NULL );
				p = new BYTE[size];
				::ReadFile( hFile, p, size, &dw, NULL );
				if ( p_kiff->LoadFromMemory( p, size ) )
#else
				if ( p_kiff->LoadFromFile( cfile->GetFilePath() ) )
#endif
				{
					int x, y, w, h, w3, yw3, yw33;
					DWORD pixel;
					LPBYTE p;
					p_kiff->GetSize( &w, &h );
					m_DIB.Create( w, h, 24, BI_RGB );
					p = (LPBYTE) m_DIB.GetBits();
					w3 = (w * 3 + 3) & ~3;
					for( y = 0; y < h; y++ )
					{
						yw3 = y * w3;
						for( x = 0; x < w; x++ )
						{
							yw33 = yw3 + x * 3;
							pixel = p_kiff->GetPixel( x, h - y - 1 );
							p[yw33 + 0] = (BYTE) (pixel >> 16);
							p[yw33 + 1] = (BYTE) (pixel >> 8);
							p[yw33 + 2] = (BYTE) pixel;
						}
					}
					m_type = TYPE_KIFF;
#if USE_MEMORY
					delete p;
					::CloseHandle( hFile );
#endif
				}
				else m_type = TYPE_NONE;
			}
			else
			{
				m_zooming = 100;
				m_windowSize.cx = 400;
				m_windowSize.cx = 300;
				m_type = TYPE_NONE;
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CKiffrefDoc diagnostics

#ifdef _DEBUG
void CKiffrefDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CKiffrefDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CKiffrefDoc commands

void CKiffrefDoc::ReSize()
{
	if ( m_type != TYPE_NONE )
	{
		POSITION pos = GetFirstViewPosition();
		CKiffrefView *pView;
		pView = (CKiffrefView*) GetNextView( pos );

		RECT clientWindowRect;
		RECT clientViewRect;
		SIZE dibSize;
		CWnd *pWnd;
		pWnd = pView->GetParent();

		dibSize.cx = m_DIB.GetWidth() * m_zooming / 100;
		dibSize.cy = m_DIB.GetHeight() * m_zooming / 100;

		pWnd->GetWindowRect(&clientWindowRect);
		pView->GetClientRect(&clientViewRect);
		m_windowSize.cx = dibSize.cx + (clientWindowRect.right - clientWindowRect.left) - clientViewRect.right;
		m_windowSize.cy = dibSize.cy + (clientWindowRect.bottom - clientWindowRect.top) - clientViewRect.bottom;
		pWnd->SetWindowPos( pWnd , 0, 0, m_windowSize.cx, m_windowSize.cy, SWP_NOMOVE | SWP_NOZORDER | SWP_SHOWWINDOW );
	}
}

BOOL CKiffrefDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	
	ReSize();

	return TRUE;
}

void CKiffrefDoc::ViewReDraw()
{
	POSITION pos = GetFirstViewPosition();
	CView *pview;
	while( pos != NULL )
	{
		pview = GetNextView( pos );
		pview->Invalidate();
	}
}

void CKiffrefDoc::OnZoom50() 
{
	m_zooming = 50;
	ViewReDraw();
	ReSize();
}

void CKiffrefDoc::OnZoom100() 
{
	m_zooming = 100;
	ViewReDraw();
	ReSize();
}

void CKiffrefDoc::OnZoom200() 
{
	m_zooming = 200;
	ViewReDraw();
	ReSize();
}

void CKiffrefDoc::OnUpdateZoom50(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( (m_zooming == 50) );
	pCmdUI->Enable( (BOOL) m_DIB.GetBits() );
}

void CKiffrefDoc::OnUpdateZoom100(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( (m_zooming == 100) );
	pCmdUI->Enable( (BOOL) m_DIB.GetBits() );
}

void CKiffrefDoc::OnUpdateZoom200(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( (m_zooming == 200) );
	pCmdUI->Enable( (BOOL) m_DIB.GetBits() );
}

void CKiffrefDoc::OnFileOpen() 
{
	char dropdown_files[] = {
		"Bitmap (*.bmp)|*.bmp|"
		"Kiff (*.kiff;*.kif)|*.kiff;*.kif|"
	};
	CFileDialog FileDlg( TRUE, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, dropdown_files );
	if ( FileDlg.DoModal() == IDOK )
	{
		CString FileName = FileDlg.GetPathName();
		OnOpenDocument( FileName );
	}
}

void CKiffrefDoc::OnFileSave() 
{
	switch( m_type )
	{
	case TYPE_BITMAP:
		{
			int mode, factor, matrix;
			char dropdown_files[] = { "Kiff (*.kif;*.kiff)|*.kif;*.kiff|" };
			CFileDialog FileDlg( FALSE, "kif", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, dropdown_files );
			if ( FileDlg.DoModal() == IDOK )
			{
				CString FileName = FileDlg.GetPathName();
				CKiffSettingsDlg kiffDlg;
				if ( kiffDlg.DoModal() == IDOK )
				{
					LPBYTE p;
					DWORD pixel;
					int x, y, w, h, w3, yw3, yw33, len;
					kiffDlg.GetParameter( &mode, &factor, &matrix );
					w = m_DIB.GetWidth();
					h = m_DIB.GetHeight();
					len = m_DIB.GetBitSize();
					p = new BYTE[len];
					p_kiff->SetSize( w, h );
					m_DIB.CopyBits( p );
					w3 = (w * 3 + 3) & ~3;
					for( y = 0; y < h; y++ )
					{
						yw3 = y * w3;
						for( x = 0; x < w; x++ )
						{
							yw33 = yw3 + x * 3;
							pixel =   (p[yw33 + 0] << 16)
									+ (p[yw33 + 1] << 8)
									+ (p[yw33 + 2]);
							p_kiff->SetPixel( x, h - y - 1, pixel );
						}
					}
					p_kiff->SaveToFile( FileName, mode, factor, matrix );
					delete p;
				}
			}
		}
		break;
	case TYPE_KIFF:
		{
			char dropdown_files[] = { "Bitmap (*.bmp)|*.bmp|" };
			CFileDialog FileDlg( FALSE, "bmp", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, dropdown_files );
			if ( FileDlg.DoModal() == IDOK )
			{
				BITMAPFILEHEADER head1;
				BITMAPINFOHEADER head2;
				LPBYTE pline, pline2;
				int x, y, pitch;
				int Width, Height, WidthEx, HeightEx;
				DWORD writed, pixel;
				CString FileName = FileDlg.GetPathName();

				HANDLE h_file = ::CreateFile( FileName, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, NULL, NULL );
				if ( h_file )
				{
					p_kiff->GetSize( &Width, &Height );
					p_kiff->GetExSize( &WidthEx, &HeightEx );
					pitch = ((Width * 3 + 3) & (~3));
					head1.bfType = 0x4d42;
					head1.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + Height * pitch;
					head1.bfReserved1 = head1.bfReserved2 = 0;
					head1.bfOffBits = 0x36;

					ZeroMemory( &head2, sizeof(BITMAPINFOHEADER) );
					head2.biSize = sizeof(BITMAPINFOHEADER);
					head2.biWidth = Width;
					head2.biHeight = Height;
					head2.biPlanes = 1;
					head2.biBitCount = 24;
					head2.biCompression = BI_RGB;

					WriteFile( h_file, &head1, sizeof(BITMAPFILEHEADER), &writed, NULL );
					WriteFile( h_file, &head2, sizeof(BITMAPINFOHEADER), &writed, NULL );

					pline = new BYTE[pitch];
					ZeroMemory( pline, pitch );
					for( y = 0; y < Height; y++ )
					{
						pline2 = pline;
						for( x = 0; x < Width; x++ )
						{
							pixel = p_kiff->GetPixel( x, Height - y - 1 );
							pline2[0] = (BYTE) (pixel >> 16);
							pline2[1] = (BYTE) (pixel >> 8);
							pline2[2] = (BYTE) (pixel);
							pline2 += 3;
						}
						WriteFile( h_file, pline, pitch, &writed, NULL );
					}
					delete pline;
					CloseHandle( h_file );
				}
			}
		}
		break;
	default:
		break;
	}
}
