/*--------------------------------------------------------------------

          Sample source code for KIFF.DLL / Static KIFF.LIB

--------------------------------------------------------------------*/
/*--------------------------------------------------------------------

              Kt Interchange picture File Format CODEC
                    Copyright (c) KT-Lab. 1996-2000

                     Author: Yoshikazu Kita
                             Shinji Chiba

--------------------------------------------------------------------*/
#include <windows.h>
#include <stdio.h>
#include "kiff.h"

// Undefine here if you would like to use static kiff library.
#define KIFF_DLL

#ifdef KIFF_DLL
HINSTANCE hInst;
KIFF* (*KiffProc)();
VOID (*KiffFree)( KIFF* );
KIFF* p_kiff;
#else
#pragma comment( lib, "lib/kiff" )
extern KIFF ckiff;
#endif

char getopt( char *argv )
{
	if ( argv[0] == '-' || argv[0] == '/' )
	{
		if ( argv[1] >= '0' && argv[1] <= '9' ) return atoi( &argv[1] );
		else
		{
			if ( argv[1] == 'c' || argv[1] == 'C' ) return 'c';
			if ( argv[1] == 'j' || argv[1] == 'J' ) return 'j';
			if ( argv[1] == 'k' || argv[1] == 'K' ) return 'k';
			if ( argv[1] == 'r' || argv[1] == 'R' ) return 'r';
			if ( argv[1] == 'y' || argv[1] == 'Y' ) return 'y';
		}
		return argv[1];
	}
	return -1;
}

void usage( char *argv )
{
	char str[MAX_PATH];
	char *p_str;
	DWORD version;

	p_str = strrchr( argv, '\\' );
	if ( p_str ) strcpy( str, p_str + 1 );
	else strcpy( str, argv );
	printf( "Usage: %s <de> [-/0~15][-/cjk][-/ry] <infile> [outfile]\n\n"
			"Commands:\n"
			"      <d> Decode kiff to bmp      <e> Encode bmp to kiff\n\n"
			"Options: (*=Default)\n"
			" Q-Factor: -0~15 Compression level(*=0 Best Quality)\n"
			" Q-Matrix:  -c  CG,Animation      -j  JPEG standard      -k  Kiff original(*)\n"
			" Mode:      -r  RGB compression   -y  YUV compression(*)\n\n"
	, str );

#ifdef KIFF_DLL
	if ( hInst )
	{
		(FARPROC&) KiffProc = GetProcAddress( hInst, "KIFF_Constructor" );
		if ( KiffProc )
		{
			p_kiff = KiffProc();
			if ( p_kiff )
			{
				version = p_kiff->GetVersion();
				printf( "Loading... 'KIFF.DLL' version %x.%02x\n", (version >> 8), (version & 0xff) );
				(FARPROC&) KiffFree = GetProcAddress( hInst, "KIFF_Destructor" );
				if ( KiffFree ) KiffFree( p_kiff );
			}
			else printf( "not found.\n" );
		}
	}
#else
	version = ckiff.GetVersion();
	printf( "KIFF Library version %x.%02x\n", (version >> 8), (version & 0xff) );
#endif
}

void main( int argc, char **argv )
{
	char outstr[MAX_PATH+4];
	int args, mode, level, q;
	DWORD version;
	char c;

#ifdef KIFF_DLL
	hInst = LoadLibrary( "KIFF.DLL" );
#endif

	printf( "[[ KIFF Library ]] reference code.  Copyright(c) Kita-laboratory 1996-2000\n\n" );
	if ( argc < 3 ) usage( argv[0] );
	else
	{
#ifdef KIFF_DLL
		if ( ! hInst ) printf( "File not found - KIFF.DLL\n" );
		else
		{
			(FARPROC&) KiffProc = GetProcAddress( hInst, "KIFF_Constructor" );
			if ( KiffProc )
			{
				p_kiff = KiffProc();
				if ( p_kiff )
				{
					version = p_kiff->GetVersion();
					printf( "Loading... 'KIFF.DLL' version %x.%02x\n", (version >> 8), (version & 0xff) );
#else
					version = ckiff.GetVersion();
					printf( "KIFF Library version %x.%02x\n", (version >> 8), (version & 0xff) );
#endif
					level = 0;
					q = Q_KIFF_ORIGINAL;
					mode = KIFF_MODE_YUV;
					args = 2;
LOOP:
					c = getopt( argv[args] );
					if ( c != -1 )
					{
						if ( c >= 0 && c <= 15 ) level = c;
						else if ( c == 'c' ) q = Q_CG_ANIMATIONS;
						else if ( c == 'j' ) q = Q_JPEG_STANDARD;
						else if ( c == 'k' ) q = Q_KIFF_ORIGINAL;
						else if ( c == 'r' ) mode = KIFF_MODE_RGB;
						else if ( c == 'y' ) mode = KIFF_MODE_YUV;
						args++;
						if ( argc <= args )
						{
							usage( argv[0] );
							exit( 0 );
						}
						goto LOOP;
					}
					if ( argv[1][0] == 'd' )
					{
						if ( argc == (args + 1) )
						{
							strcpy( outstr, argv[args] );
							strcat( outstr, ".bmp" );
						}
						else strcpy( outstr, argv[args + 1] );
#ifdef KIFF_DLL
						if ( ! p_kiff->ConvertKiffToBmp( argv[args], outstr ) )
#else
						if ( ! ckiff.ConvertKiffToBmp( argv[args], outstr ) )
#endif
							printf( "error: This file is not the KIF Format??\n" );
					}
					else
					{
						if ( argc == (args + 1) )
						{
							strcpy( outstr, argv[args] );
							strcat( outstr, ".kif" );
						}
						else strcpy( outstr, argv[args + 1] );
						printf( "Output=%s\n", outstr );
#ifdef KIFF_DLL
						if ( ! p_kiff->ConvertBmpToKiff( argv[args], outstr, mode, level, q ) )
#else
						if ( ! ckiff.ConvertBmpToKiff( argv[args], outstr, mode, level, q ) )
#endif
							printf( "error: not the 16/24/32 Bitmap?!?\n" );
					}
#ifdef KIFF_DLL
					(FARPROC&) KiffFree = GetProcAddress( hInst, "KIFF_Destructor" );
					if ( KiffFree ) KiffFree( p_kiff );
				}
			}
			else printf( "GetProc error!\n" );
			FreeLibrary( hInst );
		}
#endif
	}
}
